unit uColorGrid;

interface

uses
  Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Grids;

type

  TCellColor = record
    TextColor: TColor; // ֵɫ
    BackGroundColor: TColor; // ӱɫ
    ColorChanged: boolean;
  end;

  TCellColorArr = array of array of TCellColor;

  TStringGrid = class(Vcl.Grids.TStringGrid)
  private
    FCellColorArr: TCellColorArr; // ¼Ԫɫ
    function GetCellsColor(ACol, ARow: integer): TCellColor;
    procedure SetCellsColor(ACol, ARow: integer; const Value: TCellColor);
    procedure InitCellsColor(ACol, ARow: integer);
  protected
    procedure SizeChanged(OldColCount, OldRowCount: Longint); override; // ڴ˹еɫ¼ĴС
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
  public
    constructor Create(AOwner: TComponent); override;
    property CellsColor[ACol, ARow: integer]: TCellColor read GetCellsColor write SetCellsColor;
  end;

implementation

constructor TStringGrid.Create(AOwner: TComponent);
begin
  inherited;
  InitCellsColor(ColCount, RowCount);
end;

procedure TStringGrid.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);
var
  CC: TCellColor;
  oldBrush: Tbrush;
  oldPen: tpen;
  oldFont: tfont;
begin
  CC := FCellColorArr[ARow, ACol];
  if CC.ColorChanged then
  begin
    oldBrush := self.Canvas.Brush;
    oldPen := self.Canvas.Pen;
    oldFont := self.Canvas.Font;
    Canvas.Font.Color := CC.TextColor; // ɫ
    Canvas.Brush.Color := CC.BackGroundColor; // ɫ
    inherited;
    Canvas.Brush := oldBrush;
    Canvas.Pen := oldPen;
    Canvas.Font := oldFont;
  end
  else
    inherited;
end;

function TStringGrid.GetCellsColor(ACol, ARow: integer): TCellColor;
begin
  result := FCellColorArr[ARow, ACol];
end;

procedure TStringGrid.InitCellsColor(ACol, ARow: integer);
var
  i, j: integer;
begin
  setlength(FCellColorArr, ARow);
  for i := 0 to ARow - 1 do
  begin
    setlength(FCellColorArr[i], ACol);
    for j := 0 to ACol - 1 do
    begin
      FCellColorArr[i, j].ColorChanged := false; //ʼ
    end;
  end;

end;

procedure TStringGrid.SetCellsColor(ACol, ARow: integer; const Value: TCellColor);
begin
  FCellColorArr[ARow, ACol] := Value;
  FCellColorArr[ARow, ACol].ColorChanged := true;
end;

procedure TStringGrid.SizeChanged(OldColCount, OldRowCount: Longint);
begin
  inherited;
  InitCellsColor(ColCount, RowCount);
end;

end.
